class Day
{
public:
    /**
     * Public constructor.
     * Construct a day with a given year, month, and day
     * of the Julian/Gregorian calendar. The Julian calendar
     * is used for all days before October 15, 1582.
     * @param y a year != 0
     * @param m a month between 1 and 12
     * @param d a date between 1 and 31
     */
    Day(int y, int m, int d)
    {
        julian = to_julian(y, m, d);
    }

    /**
     * Getter.
     * @return the year of this day.
     */
    int get_year();

    /**
     * Getter.
     * @return the month of this day.
     */
    int get_month();

    /**
     * Getter.
     * @return the date of this day.
     */
    int get_date();

    /**
     * Return the day that is a given number of days away from this day.
     * @param n the number of days, can be negative.
     * @return the day.
     */
    Day *add_days(int n);

    /**
     * Return the number of days between this day and another day.
     * @param other the other day.
     * @return the number of days (> 0 if this day comes before)
     */
    int days_from(Day *other);

private:
    int julian;  // Julian date

    /**
     * Private constructor.
     * Construct a day with a given Julian date.
     * @param j the Julian date.
     */
    Day(const int j) { julian = j; }

    /**
     * Compute the Julian date of a given day.
     * Positive year for CE, negative for BCE. The year after 1 BCE is 1 CE.
     * Algorithm from Press et al., Numerical Recipes in C, 2nd ed.,
     * Cambridge University Press, 1992.
     * @param year the year.
     * @param month the month.
     * @param date the day of the month.
     */
    static int to_julian(const int year, const int month, const int date);

    /**
     * Convert a Julian date to a calendar date.
     * Algorithm from Press et al., Numerical Recipes in C, 2nd ed.,
     * Cambridge University Press, 1992.
     * @param j the Julian day number.
     * @param ymd the result array, [0] = year, [1] = month, [2] = day of month.
     */
    static void from_julian(const int j, int ymd[]);
};
